@@ -34,6 +34,8 @@ The variables used when configuring are :
 * datadir    : install path for the data, eg: /usr/local/share
 * libdir     : install path for the libs, eg: /usr/lib
 
+* libsuffix  : for those who need /usr/lib64 and the like ..
+
 * kdeincludes: path to the kde includes (/usr/include/kde on debian, ...)
 * qtincludes : same punishment, for qt includes (/usr/include/qt on debian, ...)
 
@@ -46,7 +48,8 @@ eg: scons configure libdir=/usr/local/lib qtincludes=/usr/include/qt
 BOLD   ="\033[1m"
 RED    ="\033[91m"
 GREEN  ="\033[92m"
-YELLOW ="\033[93m"
+#YELLOW ="\033[93m"
+YELLOW ="\033[1m"
 CYAN   ="\033[96m"
 NORMAL ="\033[0m"
 
@@ -61,11 +64,15 @@ def detect_kde(env):
 	execprefix  = env['ARGS'].get('execprefix', None)
 	datadir     = env['ARGS'].get('datadir', None)
 	libdir      = env['ARGS'].get('libdir', None)
+	libsuffix   = env['ARGS'].get('libsuffix', '')
 	kdeincludes = env['ARGS'].get('kdeincludes', None)
 	kdelibs     = env['ARGS'].get('kdelibs', None)
 	qtincludes  = env['ARGS'].get('qtincludes', None)
 	qtlibs      = env['ARGS'].get('qtlibs', None)
 
+	if libdir:
+		libdir = libdir+libsuffix
+
 	## Detect the kde libraries
 	print "Checking for kde-config           : ",
 	kde_config = os.popen("which kde-config 2>/dev/null").read().strip()
@@ -94,7 +101,7 @@ def detect_kde(env):
 	if qtdir:
 		print GREEN + "qt is in " + qtdir + NORMAL
 	else:
-		m = re.search('(.*)/lib/libqt.*', os.popen('ldd `kde-config --expandvars --install lib`' + '/libkdeui.so | grep libqt').read().strip().split()[2])
+		m = re.search('(.*)/lib/libqt.*', os.popen('ldd `kde-config --expandvars --install lib`' + '/libkdeui.so.4 | grep libqt').read().strip().split()[2])
 		if m:
 			qtdir = m.group(1)
 			print YELLOW + "qt was found as " + m.group(1) + NORMAL
@@ -178,16 +185,17 @@ def detect_kde(env):
 		if not datadir:
 			datadir = prefix  + "/share"
 		if not libdir:
-			libdir = execprefix + "/lib"
+			libdir = execprefix + "/lib"+libsuffix
 
 		subst_vars = lambda x: x.replace('${exec_prefix}', execprefix).replace('${datadir}', 
 			datadir).replace('${libdir}', libdir)
+		debian_fix = lambda x: x.replace('/usr/share', '${datadir}')
 		env['KDEBIN']   = subst_vars(os.popen('kde-config --install exe').read().strip())
 		env['KDEAPPS']  = subst_vars(os.popen('kde-config --install apps').read().strip())		
 		env['KDEDATA']  = subst_vars(os.popen('kde-config --install data').read().strip())
 		env['KDEMODULE']= subst_vars(os.popen('kde-config --install module').read().strip())
 		env['KDELOCALE']= subst_vars(os.popen('kde-config --install locale').read().strip())
-		env['KDEDOC']   = subst_vars(os.popen('kde-config --install html').read().strip())
+		env['KDEDOC']   = subst_vars( debian_fix(os.popen('kde-config --install html').read().strip()) )
 		env['KDEKCFG']  = subst_vars(os.popen('kde-config --install kcfg').read().strip())
 		env['KDEXDG']   = subst_vars(os.popen('kde-config --install xdgdata-apps').read().strip())
 		env['KDEMENU']  = subst_vars(os.popen('kde-config --install apps').read().strip())
@@ -227,7 +235,24 @@ def detect_kde(env):
 
 
 def generate(env):
-	""" Set up the qt and kde environment and builders - the moc part is difficult to understand """
+	""""Set up the qt and kde environment and builders - the moc part is difficult to understand """
+
+	env.Help("""
+"""+BOLD+
+"""*** KDE options ***
+-------------------"""
++NORMAL+"""
+"""+BOLD+"""* prefix     """+NORMAL+""": base install path,         ie: /usr/local
+"""+BOLD+"""* execprefix """+NORMAL+""": install path for binaries, ie: /usr/bin
+"""+BOLD+"""* datadir    """+NORMAL+""": install path for the data, ie: /usr/local/share
+"""+BOLD+"""* libdir     """+NORMAL+""": install path for the libs, ie: /usr/lib
+"""+BOLD+"""* libsuffix  """+NORMAL+""": suffix of libraries on amd64, ie: 64, 32
+"""+BOLD+"""* kdeincludes"""+NORMAL+""": path to the kde includes (/usr/include/kde on debian, ...)
+"""+BOLD+"""* qtincludes """+NORMAL+""": same punishment, for qt includes (/usr/include/qt on debian, ...)
+"""+BOLD+"""* kdelibs    """+NORMAL+""": path to the kde libs, for linking the programs
+"""+BOLD+"""* qtlibs     """+NORMAL+""": same punishment, for qt libraries
+ie: """+BOLD+"""scons configure libdir=/usr/local/lib qtincludes=/usr/include/qt
+"""+NORMAL)
 
 	import os.path
 	import re
@@ -507,7 +532,16 @@ def generate(env):
 	
 		#    find_file(kcfgfile, (source[0].get_dir(),) ,SCons.Node.FS.default_fs)
 		if len(source) <2:
-			kcfgfilename = os.popen("cat "+str(source[0])+"|grep File|sed 's/File.*=//i'").read().rstrip()
+			if not os.path.isfile(str(source[0])):
+				print RED+'kcfg file given'+str(source[0])+' does not exist !'+NORMAL
+				return target, source
+			kfcgfilename = ""
+			kcfgFileDeclRx = re.compile("^[fF]ile\s*=\s*(.+)\s*$")
+			for line in file(str(source[0]), "r").readlines():
+				match = kcfgFileDeclRx.match(line.strip())
+				if match:
+					kcfgfilename = match.group(1)
+					break
 			source.append( str(source[0].get_dir())+'/'+kcfgfilename )
 		return target, source
 
@@ -614,8 +648,18 @@ def generate(env):
 	import SCons.Util
 	skel_ext = [".skel", ".SKEL"]
 	def KDEfiles(target, source, env):
-		""" Returns a list of files for scons (handles kde tricks like .skel) """
+		"""
+		Returns a list of files for scons (handles kde tricks like .skel) 
+		It also makes custom checks against double includes like : ['file.ui', 'file.cpp']
+		(file.cpp is already included because of file.ui)
+		"""
 		src=[]
+		ui_files=[]
+		kcfg_files=[]
+		skel_files=[]
+		other_files=[]
+
+		# For each file, check wether it is a dcop file or not, and create the complete list of sources
 		for file in source:
 			bs  = SCons.Util.splitext(file)[0]
 			ext = SCons.Util.splitext(file)[1]
@@ -624,51 +668,87 @@ def generate(env):
 				src.append(bs+'_skel.cpp')
 			else:
 				src.append(file)
+
+			if ext == '.ui':
+				ui_files.append(bs)
+			elif ext == '.kcfgc':
+				kcfg_files.append(bs)
+			elif ext == '.skel':
+				skel_files.append(bs)
+			else:
+				other_files.append(bs)	
+		
+		# Now check against newbie errors
+		for file in ui_files:
+			for ofile in other_files:
+				if ofile == file:
+					print RED+"WARNING: You have included "+file+".ui and another file of the same prefix"+NORMAL
+					print "Files generated by uic (file.h, file.cpp must not be included"
+		for file in kcfg_files:
+			for ofile in other_files:
+				if ofile == file:
+					print RED+"WARNING: You have included "+file+".kcfg and another file of the same prefix"+NORMAL
+					print "Files generated by kconfig_compiler (settings.h, settings.cpp) must not be included"
+		#for file in skel_files:
+		#	for ofile in other_files:
+		#		if ofile == file:
+		#			print RED+"WARNING: source contain "+file+".skel and another file of the same prefix"+NORMAL
+		#			print "Files generated automatically from .skel file must not be included (file.h, file.idl)"
+		
 		return src
 
-	def KDEprogram(target, source, lenv):
-		""" Makes a kde program """
-		src = KDEfiles(target, source, lenv)
-		lenv.Program(target, src)
-		if 'install' in env['TARGS']:
-			env.Alias('install', env.Install( env['KDEBIN'], target ) )
-
-	def KDEshlib(target, source, lenv):
-		""" Makes a shared library for kde (.la file for klibloader) """
-		src = KDEfiles(target, source, lenv)
-		lenv.SharedLibrary( target, src )
-		lenv.LaFile( target, target+'.so' )
-		if 'install' in env['TARGS']:
-			env.Alias('install', env.Install( env['KDEMODULE'], target+'.so' ) )
-			env.Alias('install', env.Install( env['KDEMODULE'], target+'.la' ) )
-
-#	def KDEstaticlib(target, source, lenv):
-#		""" Makes a static library for kde - in practice you should not use static libraries 
-#		1. they take more memory than shared ones
-#		2. makefile.am needed it because of stupid limitations
-#		(cannot handle sources in separate folders - takes extra processing) """
-#		src = KDEfiles(target, source, lenv)
-#		lenv.StaticLibrary( target, src )
-#		if 'install' in env['TARGS']:
-#			env.Alias('install', env.Install( env['KDEMODULE'], target+'.a' ) )
-
-	def KDEaddlibs(libs, env):
-		""" Helper function """
-		env.AppendUnique(LIBS = libs)
-
-	def KDEaddpaths(paths, env):
-		""" Helper function """
-		env.AppendUnique(CPPPATH = paths)
+	# Special trick for installing rpms ...
+	env['DESTDIR']=''
+	if 'install' in env['TARGS'] and os.environ.has_key('DESTDIR'):
+		env['DESTDIR']=os.environ['DESTDIR']+'/'
+		print CYAN+'** Enabling DESTDIR for the project **' + NORMAL + env['DESTDIR']
 
 	def KDEinstall(path, file, lenv):
 		""" Quick wrapper """
-		if 'install' in env['TARGS']:
-			env.Alias('install', env.Install( path, file ) )
+		if 'install' in lenv['TARGS']:
+			lenv.Alias('install', lenv.Install( lenv['DESTDIR']+path, file ) )
 
 	def KDEinstallas(destfile, file, lenv):
 		""" Quick wrapper """
-		if 'install' in env['TARGS']:
-			env.Alias('install', env.InstallAs( destfile, file ) )
+		if 'install' in lenv['TARGS']:
+			lenv.Alias('install', lenv.InstallAs( lenv['DESTDIR']+destfile, file ) )
+
+	def KDEprogram(target, source, lenv):
+		""" Makes a kde program 
+		The program is installed except if one sets env['NOAUTOINSTALL'] """
+		src = KDEfiles(target, source, lenv)
+		lenv.Program(target, src)
+		if not lenv.has_key('NOAUTOINSTALL'):
+			KDEinstall(env['KDEBIN'], target, lenv)
+
+	def KDEshlib(target, source, lenv):
+		""" Makes a shared library for kde (.la file for klibloader)
+		The library is installed except if one sets env['NOAUTOINSTALL'] """
+		src = KDEfiles(target, source, lenv)
+		lenv.SharedLibrary( target, src )
+		lenv.LaFile( target, target+'.so' )
+		if not lenv.has_key('NOAUTOINSTALL'):
+			KDEinstall(env['KDEMODULE'], target+'.so', lenv)
+			KDEinstall(env['KDEMODULE'], target+'.la', lenv)
+
+	def KDEstaticlib(target, source, lenv):
+		""" Makes a static library for kde - in practice you should not use static libraries 
+		1. they take more memory than shared ones
+		2. makefile.am needed it because of stupid limitations
+		(cannot handle sources in separate folders - takes extra processing) """
+		src = KDEfiles(target, source, lenv)
+		lenv.StaticLibrary( target, src )
+#		# do not install static libraries
+#		if not lenv.has_key('NOAUTOINSTALL'):
+#			KDEinstall(env['KDEMODULE'], target+'.a', lenv)
+
+	def KDEaddlibs(libs, lenv):
+		""" Helper function """
+		lenv.AppendUnique(LIBS = libs)
+
+	def KDEaddpaths(paths, lenv):
+		""" Helper function """
+		lenv.AppendUnique(CPPPATH = paths)
 
 	def KDElang(transfiles, lenv):
 		""" Process translations (.po files) in a po/ dir """
@@ -684,8 +764,9 @@ def generate(env):
 		""" Install the documentation """
 		if not lenv['APPNAME']:
 			print "define lenv['APPNAME'] before using KDEdoc !!"
-			return
+			env.Exit(1)
 		KDEinstall( lenv['KDEDOC']+'/'+lang+'/'+lenv['APPNAME'], file, lenv )
 
 	# Export variables so that sconscripts in subdirectories can use them
 	env.Export("KDEprogram KDEshlib KDEaddpaths KDEaddlibs KDEinstall KDEinstallas KDElang KDEdoc")
+
