@@ -10,7 +10,6 @@
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
- *                                                                         *
  ***************************************************************************/
 
 #include <qpoint.h>
@@ -22,6 +21,7 @@
 #include <qpixmap.h>
 #include <qwmatrix.h>
 
+#include <kurldrag.h>
 #include <kmessagebox.h>
 #include <klocale.h>
 #include <kdebug.h>
@@ -35,9 +35,8 @@
 
 #include "settings.h"
 #include "aux.h"
-#include "datastruct/DDataItem.h"
-#include "datastruct/DDataControl.h"
-
+#include "DDataItem.h"
+#include "DDataControl.h"
 #include "DCanvasPos.h"
 
 // z position for link threads
@@ -115,7 +114,7 @@ DCanvasView::~DCanvasView()
 		if (lst.count() == 0)
 			break;
 
-		for (int i=0; i<lst.count(); i++)
+		for (unsigned int i=0; i<lst.count(); i++)
 		{
 			if (lst[i]->rtti() != DCanvasItem::Rtti_DCanvasItem)
 				continue;
@@ -192,16 +191,13 @@ void DCanvasView::contentsMousePressEvent(QMouseEvent* me)
 
 	// set the rubberband for selecting items
 	m_canSelect = false;
-	if (newitem == DItem::NOITEM)
-		m_canSelect = true;
+	if (newitem == DItem::NOITEM) m_canSelect = true;
 
-	//
 	if (me->button() == LeftButton)
 	{
 		if (act_point == m_currentAction)
 		{
-			if ( ! m_multiSelect)
-				deselectAll();
+			if ( ! m_multiSelect) deselectAll();
 
 			if ( newitem != DItem::NOITEM )
 			{
@@ -346,6 +342,9 @@ void DCanvasView::contentsMousePressEvent(QMouseEvent* me)
 			deselectAll();
 		}
 
+		if (oneItemIsSelected()) dataTree()->setItemSelected( m_selectedList[0], this );
+		else dataTree()->setItemSelected( DItem::NOITEM, this );
+
 		if (m_readWrite)
 		{
 			m_selectedlink = NULL;
@@ -376,19 +375,11 @@ void DCanvasView::contentsMousePressEvent(QMouseEvent* me)
 
 void DCanvasView::addItemToSelection( int id )
 {
-	if (id == DItem::NOITEM)
-		return;
-
-	if (!isRegistered(id))
-		return;
-
-	if (m_selectedList.count() == 0)
-		m_lastitemtoselect = id;
-
+	if (id == DItem::NOITEM) return;
+	if (!isRegistered(id)) return;
+	if (m_selectedList.count() == 0) m_lastitemtoselect = id;
 	canvasItem(id)->setSelected(true);
-
-	if (m_selectedList.contains(id) == 0)
-		m_selectedList.push_back(id);
+	if (m_selectedList.contains(id) == 0) m_selectedList.push_back(id);
 }
 
 void DCanvasView::contentsMouseDoubleClickEvent(QMouseEvent* me)
@@ -404,6 +395,7 @@ void DCanvasView::contentsMouseDoubleClickEvent(QMouseEvent* me)
 		{
 			if (newitem != DItem::NOITEM)
 			{
+				dataTree()->newDelta();
 				dataTree()->setOrphan( newitem );
 			}
 			else
@@ -418,6 +410,7 @@ void DCanvasView::contentsMouseDoubleClickEvent(QMouseEvent* me)
 					if ( (*it)->rtti() == DCanvasLink::Rtti_DCanvasLink )
 					{
 						DCanvasLink* link = (DCanvasLink*) (*it);
+						dataTree()->newDelta();
 						dataTree()->setOrphan( link->IdOrig() );
 						break;
 					}
@@ -432,6 +425,7 @@ void DCanvasView::contentsMouseDoubleClickEvent(QMouseEvent* me)
 		}
 		else
 		{
+			dataTree()->newDelta();
 			dataTree()->createItem( newitem );
 		}
 	}
@@ -484,15 +478,13 @@ void DCanvasView::contentsMouseMoveEvent(QMouseEvent* me)
 			{
 				DCanvasItem *citem = canvasItem(*m_it);
 
-				if (!citem)
-					return;
+				if (!citem) return;
 
-				if (!citem->isSelected())
-					citem->setSelected(true);
+				if (!citem->isSelected()) citem->setSelected(true);
 
 				citem->moveOffset(p);
 
-				DDataItem *item = dataItem( *m_it );
+				//DDataItem *item = dataItem( *m_it );
 				updateObjLinks( *m_it );
 			}
 
@@ -504,8 +496,7 @@ void DCanvasView::contentsMouseMoveEvent(QMouseEvent* me)
 	// in link mode, draw the thread for linking items
 	if (m_currentAction == act_link)
 	{
-		if (m_rubberLine)
-			m_rubberLine->setPoints( m_moveStart.x(), m_moveStart.y(), p.x(), p.y());
+		if (m_rubberLine) m_rubberLine->setPoints( m_moveStart.x(), m_moveStart.y(), p.x(), p.y());
 	}
 
 	// in scroll mode .. well, scroll :)
@@ -520,7 +511,7 @@ void DCanvasView::contentsMouseReleaseEvent(QMouseEvent* me)
 	m_pressed = false;
 
 	m_rubberRect->move( -10, -10 );
-	m_rubberRect->setSize( 10, 10);
+	m_rubberRect->setSize( 10, 10 );
 
 	// even with invalidate() there are redrawing errors .. :-/
 	// m_rubberRect is a QCanvasRectangle though
@@ -541,8 +532,16 @@ void DCanvasView::contentsMouseReleaseEvent(QMouseEvent* me)
 			{
 				if (m_startItem->Id() != id)
 				{
-					if (!m_ctrlIsPressed) dataTree()->linkItems( m_startItem->Id(), id );
-					else dataTree()->requestRefChange( m_startItem->Id(), id, true );
+					if (!m_ctrlIsPressed)
+					{
+						dataTree()->newDelta();
+						dataTree()->linkItems( m_startItem->Id(), id );
+					}
+					else
+					{
+						dataTree()->newDelta();
+					       	dataTree()->requestRefChange( m_startItem->Id(), id, true );
+					}
 				}
 				else
 				{
@@ -560,20 +559,43 @@ void DCanvasView::contentsMouseReleaseEvent(QMouseEvent* me)
 	{
 		if (m_lastitem != DItem::NOITEM && newitem != DItem::NOITEM && m_ctrlIsPressed)
 		{
+			dataTree()->newDelta();
 			dataTree()->linkItems( m_lastitem, newitem );
 		}
+		else if (!m_canSelect && !m_multiSelect && m_readWrite )
+		{
+			QPoint p = inverseWorldMatrix().map(me->pos());
+
+			if (m_selectedList.count() > 0) dataTree()->newDelta();
+			
+			// Foreach item in selection, add a Delta
+			QValueList<int>::iterator end = m_selectedList.end();
+                        for ( m_it = m_selectedList.begin(); m_it != end; ++m_it )
+                        {
+                                DCanvasItem *citem = canvasItem(*m_it);
+				DDataItem *ditem = dataItem(*m_it);
+                                if (!citem) return;
+
+				int dx = p.x() - m_moveStart.x();
+				int dy = p.y() - m_moveStart.y();
+				int x = ditem->x();
+				int y = ditem->y();
+				
+				dataTree()->registerMapPositionChange(*m_it, x-dx, y-dy, x, y);
+                        }
+		}
 		m_lastitem = newitem;
 	}
 
-	if (oneItemIsSelected())
-		dataTree()->setItemSelected( m_selectedList[0], this );
-	else
-		dataTree()->setItemSelected( DItem::NOITEM, this );
+	if (oneItemIsSelected()) dataTree()->setItemSelected( m_selectedList[0], this );
+	else dataTree()->setItemSelected( DItem::NOITEM, this );
+
+	emit selectionChanged();
 }
 
 void DCanvasView::contentsWheelEvent( QWheelEvent* e )
 {
-	if (!m_multiSelect)
+	if (!m_ctrlIsPressed)
 	{
 		e->ignore();
 		return;
@@ -632,8 +654,7 @@ int DCanvasView::locateItem( const QPoint& p )
 		}
 	}
 
-	if (!item)
-		return DItem::NOITEM;
+	if (!item) return DItem::NOITEM;
 
 	return item->Id();
 }
@@ -665,7 +686,8 @@ int DCanvasView::locateItemPos( const QPoint& p)
 
 void DCanvasView::moveSelObjectsUp()
 {
-	for (int i=0; i<m_selectedList.count(); i++)
+	if (m_selectedList.count() > 0) dataTree()->newDelta();
+	for (unsigned int i=0; i<m_selectedList.count(); i++)
 	{
 		DCanvasItem *item = canvasItem( m_selectedList[i] );
 		item->move(item->x(), item->y() - 30);
@@ -675,7 +697,8 @@ void DCanvasView::moveSelObjectsUp()
 
 void DCanvasView::moveSelObjectsDown()
 {
-	for (int i=0; i<m_selectedList.count(); i++)
+	if (m_selectedList.count() > 0) dataTree()->newDelta();
+	for (unsigned int i=0; i<m_selectedList.count(); i++)
 	{
 		DCanvasItem *item = canvasItem( m_selectedList[i] );
 		item->move(item->x(), item->y() + 30);
@@ -685,7 +708,8 @@ void DCanvasView::moveSelObjectsDown()
 
 void DCanvasView::moveSelObjectsLeft()
 {
-	for (int i=0; i<m_selectedList.count(); i++)
+	if (m_selectedList.count() > 0) dataTree()->newDelta();
+	for (unsigned int i=0; i<m_selectedList.count(); i++)
 	{
 		DCanvasItem *item = canvasItem( m_selectedList[i] );
 		item->move(item->x() - 30, item->y());
@@ -695,7 +719,8 @@ void DCanvasView::moveSelObjectsLeft()
 
 void DCanvasView::moveSelObjectsRight()
 {
-	for (int i=0; i<m_selectedList.count(); i++)
+	if (m_selectedList.count() > 0) dataTree()->newDelta();
+	for (unsigned int i=0; i<m_selectedList.count(); i++)
 	{
 		DCanvasItem *item = canvasItem( m_selectedList[i] );
 		item->move(item->x() + 30, item->y());
@@ -727,11 +752,8 @@ void DCanvasView::selectObjDown()
 	{
 		int newstuff = DItem::NOITEM;
 		int lastchld = item->m_lastchild;
-		if (lastchld < item->countChildren())
-			newstuff = item->childNum(lastchld);
-		else
-			newstuff = item->childNum(0);
-
+		if (lastchld < item->countChildren()) newstuff = item->childNum(lastchld);
+		else newstuff = item->childNum(0);
 
 		deselectAll();
 		addItemToSelection( newstuff );
@@ -783,6 +805,7 @@ void DCanvasView::addChild()
 {
 	if (!oneItemIsSelected()) return;
 
+	if (m_selectedList.count() > 0) dataTree()->newDelta();
 	int id = dataTree()->createItem( m_selectedList[0] );
 	tidyClean();
 	makeObjectVisible( id );
@@ -792,6 +815,7 @@ void DCanvasView::addSibling()
 {
 	if (!oneItemIsSelected()) return;
 
+	if (m_selectedList.count() > 0) dataTree()->newDelta();
 	int id = dataTree()->createItem( dataItem( m_selectedList[0] )->Parent() );
 	tidyClean();
 	makeObjectVisible( id );
@@ -803,16 +827,15 @@ void DCanvasView::keyPressEvent( QKeyEvent* e )
 	{
 		case Qt::Key_Shift:
 			// shift key is held pressed
-			m_multiSelect = true;
+			if (!m_pressed) m_multiSelect = true;
 			break;
 
 		case Qt::Key_Delete:
-			if (m_readWrite)
-				removeSelectedItems();
+			if (m_readWrite && !m_pressed) removeSelectedItems();
 			break;
 
 		case Qt::Key_Backspace:
-			if (oneItemIsSelected())
+			if (oneItemIsSelected() && !m_pressed)
 			{
 				DDataItem *item = dataItem( m_selectedList[0] );
 				int newsize = item->m_summary.length() - 1;
@@ -823,8 +846,11 @@ void DCanvasView::keyPressEvent( QKeyEvent* e )
 			break;
 
 		case Qt::Key_Control:
-			m_lastitem = DItem::NOITEM;
-			m_ctrlIsPressed = true;
+			if (!m_pressed)
+			{
+				m_lastitem = DItem::NOITEM;
+				m_ctrlIsPressed = true;
+			}
 			e->ignore();
 			break;
 
@@ -863,6 +889,12 @@ void DCanvasView::keyPressEvent( QKeyEvent* e )
 
 		default:
 
+			if (m_pressed)
+			{
+				e->ignore();
+				break;
+			}
+			
 			if (!itemsAreSelected())
 			{
 				addItemToSelection( m_lastitemtoselect );
@@ -944,11 +976,16 @@ void DCanvasView::popupMenuSel(int sel)
 			break;
 
 		case DCanvasPopup::e_discon_parent:
-			if (oneItemIsSelected()) dataTree()->setOrphan( m_selectedList[0] );
+			if (oneItemIsSelected())
+			{
+				dataTree()->newDelta();
+				dataTree()->setOrphan( m_selectedList[0] );
+			}
 			break;
 
 		case DCanvasPopup::e_discon_obj:
 			{
+				if (m_selectedList.count() > 0) dataTree()->newDelta();
 				QValueList<int>::iterator end = m_selectedList.end();
 				for ( m_it = m_selectedList.begin(); m_it != end; ++m_it )
 					dataTree()->disconnectItem( *m_it );
@@ -956,7 +993,11 @@ void DCanvasView::popupMenuSel(int sel)
 			break;
 
 		case DCanvasPopup::e_discon_subtree:
-			if (oneItemIsSelected()) dataTree()->killFamily( m_selectedList[0] );
+			if (oneItemIsSelected())
+			{
+				dataTree()->newDelta();
+				dataTree()->killFamily( m_selectedList[0] );
+			}
 			break;
 
 		case DCanvasPopup::e_tidy_all:
@@ -992,40 +1033,17 @@ void DCanvasView::popupMenuSel(int sel)
 			break;
 
 		case Settings::EnumColorMode::custom_:
-			{
-				QColor myColor;
-				int result = KColorDialog::getColor( myColor );
-
-				if ( result == KColorDialog::Accepted )
-				{
-					QValueList<int>::iterator end = m_selectedList.end();
-					for ( m_it = m_selectedList.begin(); m_it != end; ++m_it )
-					{
-						dataItem(*m_it)->setCustomColors( 
-								myColor, 
-								QColor( Qt::black ), 
-								QColor( Qt::black ) );
-						canvasItem(*m_it)->updatePos();
-					}
-				}
-			}
+			applyCustomColor();
 			break;
 
 		case Settings::EnumColorMode::default_:
-		case Settings::EnumColorMode::clear_:
-		case Settings::EnumColorMode::important_:
-		case Settings::EnumColorMode::opposition_:
-		case Settings::EnumColorMode::enumeration_:
-		case Settings::EnumColorMode::todo_:
-		case Settings::EnumColorMode::old_:
-			{
-				QValueList<int>::iterator end = m_selectedList.end();
-				for ( m_it = m_selectedList.begin(); m_it != end; ++m_it )
-				{
-					dataItem(*m_it)->setColorScheme( sel );
-					canvasItem(*m_it)->updatePos();
-				}
-			}
+		case Settings::EnumColorMode::theme1_:
+		case Settings::EnumColorMode::theme2_:
+		case Settings::EnumColorMode::theme3_:
+		case Settings::EnumColorMode::theme4_:
+		case Settings::EnumColorMode::theme5_:
+		case Settings::EnumColorMode::theme6_:
+			applyColorScheme(sel);
 			break;
 
 		case DCanvasPopup::e_discon_link:
@@ -1039,6 +1057,7 @@ void DCanvasView::popupMenuSel(int sel)
 				}
 				else if (m_selectedlink)
 				{
+					dataTree()->newDelta();
 					dataTree()->setOrphan( m_selectedlink->IdOrig() );
 					m_selectedlink = NULL;
 				}
@@ -1053,6 +1072,39 @@ void DCanvasView::popupMenuSel(int sel)
 	m_multiSelect = false;
 }
 
+void DCanvasView::applyCustomColor()
+{
+	QColor myColor;
+	int result = KColorDialog::getColor( myColor );
+
+	if ( result == KColorDialog::Accepted )
+	{
+		if (m_selectedList.count() > 0) dataTree()->newDelta();
+
+		QValueList<int>::iterator end = m_selectedList.end();
+		for ( m_it = m_selectedList.begin(); m_it != end; ++m_it )
+		{
+			dataItem(*m_it)->setCustomColors( 
+					myColor, 
+					QColor( Qt::black ), 
+					QColor( Qt::black ) );
+			canvasItem(*m_it)->updatePos();
+		}
+	}
+}
+
+void DCanvasView::applyColorScheme(int sel)
+{
+	if (m_selectedList.count() > 0) dataTree()->newDelta();
+	
+	QValueList<int>::iterator end = m_selectedList.end();
+	for ( m_it = m_selectedList.begin(); m_it != end; ++m_it )
+	{
+		dataItem(*m_it)->setColorScheme( sel );
+		canvasItem(*m_it)->updatePos();
+	}
+}
+
 void DCanvasView::settingsChanged()
 {
 	canvas()->resize(4000, 4000);
@@ -1075,16 +1127,8 @@ void DCanvasView::settingsChanged()
 
 	QPen pen;
 	pen.setStyle( Qt::DotLine );
-	if ( dataTree()->m_reverseVideo )
-	{
-		canvas()->setBackgroundColor( QColor(Qt::black) );
-		pen.setColor( QColor( Qt::white ) );
-	}
-	else
-	{
-		canvas()->setBackgroundColor( dataTree()->m_col_background );
-		pen.setColor( QColor( Qt::black ) );
-	}
+	canvas()->setBackgroundColor( dataTree()->m_col_background );
+	pen.setColor( QColor( Qt::black ) );
 	m_rubberRect->setPen( pen );
 }
 
@@ -1093,11 +1137,12 @@ void DCanvasView::removeSelectedItems()
 	disconnect(dataTree(), SIGNAL(itemSelected(int, DGuiView*)),
 			this, SLOT( updateSelectedItem(int, DGuiView*)) );
 
+	if (m_selectedList.count() > 0) dataTree()->newDelta();
 	while ( m_selectedList.count() > 0 )
 	{
 		dataTree()->removeItem( m_selectedList.front() );
 	}
-
+	
 	connect(dataTree(), SIGNAL(itemSelected(int, DGuiView*)),
 			this, SLOT( updateSelectedItem(int, DGuiView*)) );
 }
@@ -1108,14 +1153,15 @@ void DCanvasView::createItem( int idx )
 	m_lastitemtoselect = idx;
 	DCanvasItem* item = new DCanvasItem(this, idx);
 
-	if (!item)
-		return;
+	if (!item) return;
 
 	registerItem(item);
 	if (dataItem(idx)->x() == 0 && dataItem(idx)->y() == 0)
+	{
+		// the change is added to the delta create automatically
+		dataTree()->registerMapPositionChange(idx, m_moveStart.x(), m_moveStart.y(), m_moveStart.x(), m_moveStart.y());
 		item->move( (double) m_moveStart.x(), (double) m_moveStart.y());
-	//else
-	// kdWarning()<<"x and y are :"<<dataItem(idx)->x()<<" "<<dataItem(idx)->y()<<endl;
+	}
 
 	item->show();
 }
@@ -1153,10 +1199,7 @@ void DCanvasView::setActionType(ActionType action)
 
 	if (m_currentAction == act_link)
 	{
-		if (Settings::reverseVideo())
-			m_rubberLine->setPen( QColor(Qt::white) );
-		else
-			m_rubberLine->setPen( QColor(Qt::black) );
+		m_rubberLine->setPen( QColor(Qt::black) );
 	}
 
 	if (m_currentAction != act_sort)
@@ -1189,16 +1232,13 @@ void DCanvasView::updateItem(int id)
 
 void DCanvasView::plug()
 {
-	if (!dataTree())
-		return;
+	if (!dataTree()) return;
 	connect(dataTree(), SIGNAL(settingsChanged()), this, SLOT(settingsChanged()) );
 	connect(dataTree(), SIGNAL(itemChanged(int)), this, SLOT(updateItem(int)) );
 	connect(dataTree(), SIGNAL(itemCreated(int)), this, SLOT(createItem(int)) );
 	connect(dataTree(), SIGNAL(itemRemoved(int)), this, SLOT(removeItem(int)) );
-	connect(dataTree(), SIGNAL(refChanged(int, int, bool)), 
-			this, SLOT(changeRef(int, int, bool)) );
-	connect(dataTree(), SIGNAL(itemSelected(int, DGuiView*)), 
-			this, SLOT(updateSelectedItem(int, DGuiView*)) );
+	connect(dataTree(), SIGNAL(refChanged(int, int, bool)), this, SLOT(changeRef(int, int, bool)) );
+	connect(dataTree(), SIGNAL(itemSelected(int, DGuiView*)), this, SLOT(updateSelectedItem(int, DGuiView*)) );
 
 	// load the document settings
 	settingsChanged();
@@ -1206,22 +1246,19 @@ void DCanvasView::plug()
 
 void DCanvasView::unplug()
 {
-	if (!dataTree())
-		return;
+	if (!dataTree()) return;
 	disconnect(dataTree(), SIGNAL(settingsChanged()), this, SLOT(settingsChanged()) );
 	disconnect(dataTree(), SIGNAL(itemChanged(int)), this, SLOT(updateItem(int)) );
 	disconnect(dataTree(), SIGNAL(itemCreated(int)), this, SLOT(createItem(int)) );
 	disconnect(dataTree(), SIGNAL(itemRemoved(int)), this, SLOT(removeItem(int)) );
-	disconnect(dataTree(), SIGNAL(refChanged(int, int, bool)),
-			this, SLOT(changeRef(int, int, bool)) );
-	disconnect(dataTree(), SIGNAL(itemSelected(int, DGuiView*)), 
-			this, SLOT( updateSelectedItem(int, DGuiView*)) );
+	disconnect(dataTree(), SIGNAL(refChanged(int, int, bool)), this, SLOT(changeRef(int, int, bool)) );
+	disconnect(dataTree(), SIGNAL(itemSelected(int, DGuiView*)), this, SLOT( updateSelectedItem(int, DGuiView*)) );
 }
 
 void DCanvasView::updateSelectedItem(int id, DGuiView* view)
 {
-	if (view == this)
-		return;
+	// somewhere else this view has requested a change
+	if (view == this) return;
 
 	if (id == DItem::NOITEM)
 	{
@@ -1240,16 +1277,16 @@ void DCanvasView::updateSelectedItem(int id, DGuiView* view)
 		deselectAll();
 		addItemToSelection(id);
 	}
+
+	// another view requested the change .. notify those watching the map
+	emit selectionChanged();
 }
 
 void DCanvasView::removeItem(int id)
 {
 	unregisterItem(id);
 	m_selectedList.remove(id);
-	if (m_itemtosort == id)
-		m_itemtosort = DItem::NOITEM;
-
-	canvas()->update();
+	if (m_itemtosort == id) m_itemtosort = DItem::NOITEM;
 }
 
 void DCanvasView::drawContents( QPainter* p, int cx, int cy, int cw, int ch)
@@ -1260,6 +1297,7 @@ void DCanvasView::drawContents( QPainter* p, int cx, int cy, int cw, int ch)
 
 void DCanvasView::tidyClean()
 {
+	dataTree()->docChanged();
 	int result = KMessageBox::warningContinueCancel(
 			this,
 			i18n("KDissert is about to reorganize the whole mindmap using the classic mindmap organization scheme. Is that what you want ?"),
@@ -1267,24 +1305,23 @@ void DCanvasView::tidyClean()
 			i18n("Reorganize"),
 			"kdissert_clean_reorganization");
 
-	if (result != KMessageBox::Continue)
-		return;
+	if (result != KMessageBox::Continue) return;
 
+	if (dataTree()->rootID() != DItem::NOITEM) dataTree()->newDelta();
+	
 	QCanvasItemList l = canvas()->allItems();
 	QCanvasItemList::iterator it;
 	QCanvasItemList::iterator end = l.end();
 	for ( it = l.begin(); it != end; ++it  )
 	{
-		if ( (*it)->rtti() != DCanvasItem::Rtti_DCanvasItem )
-			continue;
+		if ( (*it)->rtti() != DCanvasItem::Rtti_DCanvasItem ) continue;
 
 		DCanvasItem* item = (DCanvasItem*) (*it);
 		DDataItem* data = dataItem( item->Id() );
 
 		if ( data->Parent() == DItem::NOITEM )
 		{
-			if (data->countChildren() <= 0)
-				continue;
+			if (data->countChildren() <= 0) continue;
 
 			int idxmiddle = 0;
 			int cumulated = 0;
@@ -1320,21 +1357,23 @@ void DCanvasView::tidyClean()
 			for (int i=0; i<=idxmiddle; i++)
 			{
 				DCanvasItem* tmpitem = canvasItem( data->childNum(i) );
+				//tmpitem->backupPosition();
 				tmpitem->setPosition((int) item->x() - BINSPACE - tmpitem->width(), orig, false);
+				//dataTree()->registerMapPositionChange(*m_it, x-dx, y-dy, x, y);
 				orig += tmpitem->sizeH();
 			}
 
 			// process the branches on the right
 			// if  orig = data->y() + item->height()/2 + NESTING/2 - sizeright/2; else
-			if (Settings::reorgCyclicMap())
-				orig = data->y() + item->height()/2 - NESTING/2 + sizeright/2;
-			else
-				orig = data->y() + item->height()/2 + NESTING/2 - sizeright/2;
+			if (Settings::reorgCyclicMap()) orig = data->y() + item->height()/2 - NESTING/2 + sizeright/2;
+			else orig = data->y() + item->height()/2 + NESTING/2 - sizeright/2;
 
 			for (int i=idxmiddle+1; i<data->countChildren(); i++)
 			{
 				DCanvasItem* tmpitem = canvasItem( data->childNum(i) );
+				//tmpitem->backupPosition();
 				tmpitem->setPosition((int) item->x() + BINSPACE + item->width(), orig, true);
+				//dataTree()->registerMapPositionChange(*m_it, x-dx, y-dy, x, y);
 				if (Settings::reorgCyclicMap()) orig -= tmpitem->sizeH();
 				else orig += tmpitem->sizeH();
 			}
@@ -1344,6 +1383,7 @@ void DCanvasView::tidyClean()
 
 void DCanvasView::tidyCanvas()
 {
+	dataTree()->docChanged();
 	int result = KMessageBox::warningContinueCancel(
 			this,
 			i18n("KDissert is about to reorganize the whole mindmap using the smart(tm) reorganization heuristic. It can take a lot of time and give bad results on huge mindmaps. Proceed ?"),
@@ -1351,9 +1391,10 @@ void DCanvasView::tidyCanvas()
 			i18n("Reorganize"),
 			"kdissert_smart_reorganization");
 
-	if (result != KMessageBox::Continue)
-		return;
+	if (result != KMessageBox::Continue) return;
 
+	if (dataTree()->rootID() != DItem::NOITEM) dataTree()->newDelta();
+	
 	// clear the selection
 	QValueList<int>::iterator selend = m_selectedList.end();
 	for ( m_it = m_selectedList.begin(); m_it != selend; ++m_it )
@@ -1362,22 +1403,19 @@ void DCanvasView::tidyCanvas()
 	}
 	m_selectedList.clear();
 
-	int oldy=0;
-
 	QCanvasItemList l = canvas()->allItems();
 	QCanvasItemList::iterator it;
 	QCanvasItemList::iterator end = l.end();
 	for ( it = l.begin(); it != end; ++it  )
 	{
-		if ( (*it)->rtti() != DCanvasItem::Rtti_DCanvasItem )
-			continue;
+		if ( (*it)->rtti() != DCanvasItem::Rtti_DCanvasItem ) continue;
 
 		DCanvasItem* item = (DCanvasItem*) (*it);
 		DDataItem* data = dataItem( item->Id() );
 
 		if ( data->Parent() == DItem::NOITEM )
 		{
-			// TODO : the unconnected items are spent way too far
+			// TODO: the unconnected items are spent way too far
 			/*if ( data->countChildren() == 0 )
 			  {
 			  oldy += 10;
@@ -1396,16 +1434,19 @@ void DCanvasView::tidyCanvas()
 
 void DCanvasView::tidyUnclutter()
 {
+	dataTree()->docChanged();
 	//kdWarning()<<"unclutter"<<endl;
 
+	if (m_selectedList.count() > 0) dataTree()->newDelta();
+	
 	QCanvasItemList l = canvas()->allItems();
 
 	bool cluttered = true;
 	int count = 0;
 
-	while (cluttered && count < 500)
+	while (cluttered && count < 5)
 	{
-		//kdWarning()<<"iteration : "<<count<<endl;
+		//kdWarning()<<"iteration: "<<count<<endl;
 
 		// conditions for stopping
 		cluttered = false;
@@ -1416,14 +1457,12 @@ void DCanvasView::tidyUnclutter()
 		QCanvasItemList::iterator end = l.end();
 		for ( it = l.begin(); it != end; ++it  )
 		{
-			if ( (*it)->rtti() != DCanvasItem::Rtti_DCanvasItem )
-				continue;
+			if ( (*it)->rtti() != DCanvasItem::Rtti_DCanvasItem ) continue;
 
 			DCanvasItem *item = (DCanvasItem*) *it;
 			item->m_vecX = 0;
 			item->m_vecY = 0;
 
-
 			double miditem_x = item->x()+item->width()/2.;
 			double miditem_y = item->y()+item->height()/2.;
 
@@ -1432,14 +1471,14 @@ void DCanvasView::tidyUnclutter()
 			for ( tit = l.begin(); tit != end; ++tit  )
 			{
 				//kdWarning() <<"collision"<<endl;
-				if ( (*tit)->rtti() != DCanvasItem::Rtti_DCanvasItem )
-					continue;
+				if ( (*tit)->rtti() != DCanvasItem::Rtti_DCanvasItem ) continue;
 
-				// to pronounce with a German accent
+				// To pronounce with a strong German accent
 				DCanvasItem *intruder = (DCanvasItem*) *tit;
+				// If you read the line above and you found it funny, send me an email :)
+				// At least i would know that people read the source code from times to times
 
-				if (item == intruder)
-					continue;
+				if (item == intruder) continue;
 
 				double midintruder_x = intruder->x()+intruder->width()/2.;
 				double midintruder_y = intruder->y()+intruder->height()/2.;
@@ -1475,8 +1514,7 @@ void DCanvasView::tidyUnclutter()
 		// apply the changes
 		for ( it = l.begin(); it != end; ++it  )
 		{
-			if ( (*it)->rtti() != DCanvasItem::Rtti_DCanvasItem )
-				continue;
+			if ( (*it)->rtti() != DCanvasItem::Rtti_DCanvasItem ) continue;
 
 			DCanvasItem *item = (DCanvasItem*) *it;
 			item->move(item->x() + item->m_vecX, item->y() + item->m_vecY);
@@ -1505,8 +1543,7 @@ QRect DCanvasView::canvasSize()
 	QCanvasItemList::iterator end = l.end();
 	for ( it = l.begin(); it != end; ++it  )
 	{
-		if ( (*it)->rtti() != DCanvasItem::Rtti_DCanvasItem )
-			continue;
+		if ( (*it)->rtti() != DCanvasItem::Rtti_DCanvasItem ) continue;
 
 		DCanvasItem* obj = (DCanvasItem*) (*it);
 
@@ -1530,13 +1567,6 @@ QRect DCanvasView::canvasSize()
 	return QRect((int) minx, (int) miny, (int) (maxx-minx), (int) (maxy-miny) );
 }
 
-DDataItem* DCanvasView::selectedDataItem()
-{
-	if (m_selectedList.count() != 1)
-		return NULL;
-	return dataItem(m_selectedList[0]);
-}
-
 void DCanvasView::changeRef(int orig, int dest, bool add)
 {
 	DCanvasItem *item = canvasItem(orig);
@@ -1546,10 +1576,8 @@ void DCanvasView::changeRef(int orig, int dest, bool add)
 		return;
 	}
 
-	if (add)
-		item->addRef(dest);
-	else
-		item->rmRef(dest);
+	if (add) item->addRef(dest);
+	else item->rmRef(dest);
 }
 
 void DCanvasView::focusOnNextRoot()
@@ -1660,16 +1688,92 @@ void DCanvasView::dragEnterEvent( QDragEnterEvent *event )
 void DCanvasView::dropEvent( QDropEvent *event )
 {
 	QString txt;
-	if ( QTextDrag::decode(event, txt) )
+	KURL::List urllist;
+	QPoint p = inverseWorldMatrix().map( viewportToContents( event->pos() ) );
+	m_moveStart = p;
+	int id = locateItem( viewportToContents( event->pos() ) );
+	if ( KURLDrag::decode(event, urllist) )
 	{
-		// we are merging richtext - this is not perfect for the moment,
-		// however :
+		kdDebug()<<"urls dropped !! "<<urllist.count()<<endl;
+		if (! urllist.count()) return;
+
+		if (id == DItem::NOITEM)
+		{
+			dataTree()->newDelta();
+
+			id = dataTree()->createItem();
+			if (id == DItem::NOITEM) kdDebug()<<"BUG in DCanvasView::dropEvent"<<endl;
+
+			DDataItem *item = dataItem( id );
+
+			KURL url = urllist[0];
+			QString txt = url.url();
+			item->m_summary = txt.left(150);
+
+			if (urllist.count() == 1)
+			{
+				if ( txt.endsWith(".jpg", false) || txt.endsWith(".jpeg", false) || txt.endsWith(".png", false) )
+				{
+					item->loadPix(txt);
+				}
+			}
+			else
+			{
+				for (int i=0; i<urllist.count(); ++i)
+				{
+					KURL turl = urllist[i];
+
+					URLObject obj;
+					obj.m_url = turl.url();
+					item->m_urllist.append( obj );
+				}
+			}
+
+			canvasItem( id )->move( p.x(), p.y() );
+			//canvasItem( id )->updatePos();
+		}
+		else
+		{
+			DDataItem* item = dataItem( id );
+			
+			if (urllist.count() == 1)
+			{
+				QString txt = urllist[0].url();
+				if ( txt.endsWith(".jpg", false) || txt.endsWith(".jpeg", false) || txt.endsWith(".png", false) )
+				{
+					if (item->m_pix.isNull())
+						item->loadPix(txt);
+				}
+			}
+			else
+			{
+				for (int i=0; i<urllist.count(); ++i)
+				{
+					KURL turl = urllist[i];
+
+					URLObject obj;
+					obj.m_url = turl.url();
+					item->m_urllist.append( obj );
+				}
+			}
+		}
+		canvasItem( id )->updatePos();
+		dataItem( id )->setChanged();
+		dataTree()->setItemSelected( DItem::NOITEM, NULL );
+		dataTree()->setItemSelected( id, NULL );
+		dataTree()->notifyChildChange( id );
+	}
+	else if ( QTextDrag::decode(event, txt) )
+	{
+		// We are merging richtext - this is not perfect for the moment,
+		// however:
 		// 1 qtextedit is able to make the text valid 
 		// 2 we need something looking like richtext (even if buggy) for the tooltips
 		
-		int id = locateItem( viewportToContents( event->pos() ) );
 		if (id == DItem::NOITEM)
 		{
+			dataTree()->newDelta();
+			
 			id = dataTree()->createItem();
 			if (id == DItem::NOITEM) kdDebug()<<"BUG in DCanvasView::dropEvent"<<endl;
 
@@ -1686,7 +1790,6 @@ void DCanvasView::dropEvent( QDropEvent *event )
 				dataItem( id )->m_text = txt;
 			}
 
-			QPoint p = inverseWorldMatrix().map( viewportToContents( event->pos() ) );
 			canvasItem( id )->move( p.x(), p.y() );
 		}
 		else
@@ -1708,6 +1811,8 @@ void DCanvasView::dropEvent( QDropEvent *event )
 
 			dataItem( id )->m_text = wholetext;
 		}
+		dataItem( id )->setChanged();
+		dataTree()->setItemSelected( DItem::NOITEM, NULL );
 		dataTree()->setItemSelected( id, NULL );
 		dataTree()->notifyChildChange( id );
 	}
