@@ -3,24 +3,19 @@
  *   tnagy2^8@yahoo.fr                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   it under the terms of the GNU General Public License version 2        *
+ *   as published by the Free Software Foundation (see COPYING)            *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
- *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 
 #include <pwd.h>
 #include <sys/types.h>
 #include <unistd.h>
+#include <stdlib.h>
 
 #include <qdir.h>
 #include <qstring.h>
@@ -50,12 +45,16 @@
 #include "FFParser.h"
 #include "DissertParser.h"
 
-static const int document_version = 5;
+static const int document_version = 7;
 
 DDataControl::DDataControl(QObject * parent, const char * name) : QObject(parent, name)
 {
 	m_lastid = 0;
 	m_spell = NULL;
+	m_generator_count = 0;
+	m_undoredo_locked = false;
+	
+	m_reflist.setAutoDelete(true);
 
 	/// author
 	m_fname = Settings::fname();
@@ -79,7 +78,6 @@ DDataControl::DDataControl(QObject * parent, const char * name) : QObject(parent
 
 	m_canvasFont = Settings::canvasFont();
 	m_picturesOnCanvas = Settings::picturesOnCanvas();
-	m_reverseVideo = Settings::reverseVideo();
 
 	m_showDirectionSize = Settings::showDirectionSize();
 
@@ -112,7 +110,7 @@ DDataItem* DDataControl::createItemWithId(int id)
 
 	if (isRegistered(id))
 	{
-		kdWarning()<<"attempted to create an item that already exists!"<<endl;
+		kdWarning()<<"BUG: attempted to create an item that already exists!"<<endl;
 		return NULL;
 	}
 	item = new DDataItem(this, id);
@@ -121,19 +119,19 @@ DDataItem* DDataControl::createItemWithId(int id)
 	return item;
 }
 
-int DDataControl::createItem(int parent)
+int DDataControl::createItem(int parent, int optionaldefaultid)
 {
 	DDataItem* item = NULL;
 
-	while ( isRegistered(m_lastid) )
-		m_lastid++;
+	if (optionaldefaultid != DItem::NOITEM) m_lastid = optionaldefaultid;
+	while ( isRegistered(m_lastid) ) m_lastid++;
 
 	item = createItemWithId(m_lastid);
-	if (item)
-	{
-		emit itemCreated( item->Id() );
-	}
+	if (!item) kdWarning()<<"BUG DDataControl::createItem createItemWithId returned NULL"<<endl;
+	
+	addDelta( new DDeltaCreate(this, m_lastid) );
 
+	emit itemCreated( item->Id() );
 	if (parent != DItem::NOITEM)
 	{
 		linkItems(parent, item->Id());
@@ -156,10 +154,9 @@ void DDataControl::linkItems(int parent, int child)
 		return;
 	}
 
-	if (parent == child)
-		return;
+	if (parent == child) return;
 
-	// check against cycles
+	// forbid cycles
 	int iter = parent;
 	while (iter != DItem::NOITEM)
 	{
@@ -178,6 +175,8 @@ void DDataControl::linkItems(int parent, int child)
 		dataItem(parent)->addChild(child);
 		dataItem(child)->setParent(parent);
 
+		addDelta( new DDeltaLink(this, child, parent) );
+		
 		emit itemChanged(parent);
 		emit itemChanged(child);
 	}
@@ -186,12 +185,8 @@ void DDataControl::linkItems(int parent, int child)
 void DDataControl::unlinkItems(int id1, int id2)
 {
 	//kdWarning()<<"unlink "<<id1<<" "<<id2<<endl;
-
-	if (id1 == DItem::NOITEM || id2 == DItem::NOITEM)
-		return;
-
-	if (id1 == id2)
-		return;
+	if (id1 == id2) return;
+	if (id1 == DItem::NOITEM || id2 == DItem::NOITEM) return;
 
 	bool changed = false;
 
@@ -208,12 +203,14 @@ void DDataControl::unlinkItems(int id1, int id2)
 		changed = true;
 		dataItem(id1)->setParent(DItem::NOITEM);
 		dataItem(id2)->removeChild(id1);
+		addDelta( new DDeltaLink(this, id1, id2, true) );
 	}
 	else if (dataItem(id2)->Parent() == id1)
 	{
 		changed = true;
 		dataItem(id2)->setParent(DItem::NOITEM);
 		dataItem(id1)->removeChild(id2);
+		addDelta( new DDeltaLink(this, id2, id1, true) );
 	}
 
 	if (changed)
@@ -225,11 +222,11 @@ void DDataControl::unlinkItems(int id1, int id2)
 
 void DDataControl::removeItem(int id)
 {
-	//kdWarning()<<"removeItem "<<id<<endl;
-	if (!isRegistered(id) )
-		return;
+	kdWarning()<<"removeItem "<<id<<endl;
+	if (!isRegistered(id) ) return;
 
 	disconnectItem(id);
+	addDelta( new DDeltaCreate(this, id, true) );
 	unregisterItem(id);
 	emit itemRemoved( id );
 	emit itemSelected( DItem::NOITEM, NULL );
@@ -239,37 +236,53 @@ void DDataControl::removeItem(int id)
 
 void DDataControl::disconnectItem(int id)
 {
-	if (!isRegistered(id))
-		return;
+	if (!isRegistered(id)) return;
 
+	// remove the references
+	killRefs(id);
+	// disconnect the children
 	killChildren(id);
+	// disconnect the parent
 	setOrphan(id);
 
 	docChanged();
 }
 
+void DDataControl::killRefs(int id)
+{
+	if (!isRegistered(id)) return;
+	
+	QPtrList<QPoint>::iterator it = m_reflist.begin();
+	for (; it != m_reflist.end(); ++it)
+	{
+		QPoint *p = *it;
+		if (p->x() == id)
+		{
+			requestRefChange(p->x(), p->y(), false);
+			m_reflist.remove(p);
+
+		}
+		else if (p->y() == id)
+		{
+			requestRefChange(p->y(), p->x(), false);
+			m_reflist.remove(p);
+		}
+	}
+}
+
 void DDataControl::setOrphan(int child)
 {
-	if (!isRegistered(child))
-		return;
-
-	if (!child == DItem::NOITEM)
-		return;
+	if (child == DItem::NOITEM) return;
+	if (!isRegistered(child)) return;
 	int parent = dataItem(child)->Parent();
-
-	if (parent == DItem::NOITEM)
-		return;
-
-	if (child == parent)
-		return;
-
+	if (parent == DItem::NOITEM) return;
+	if (child == parent) return;
 	unlinkItems(child, parent);
 }
 
 void DDataControl::killChildren(int id)
 {
-	if (!isRegistered(id))
-		return;
+	if (!isRegistered(id)) return;
 
 	DDataItem* item = dataItem(id);
 	while (item->countChildren()>0)
@@ -280,10 +293,7 @@ void DDataControl::killChildren(int id)
 
 void DDataControl::killFamily(int id)
 {
-	while (dataItem(id)->countChildren() > 0)
-	{
-		killFamily( dataItem(id)->childNum(0) );
-	}
+	while (dataItem(id)->countChildren() > 0) killFamily( dataItem(id)->childNum(0) );
 	setOrphan(id);
 }
 
@@ -313,38 +323,31 @@ void DDataControl::clearDocument()
 
 int DDataControl::compare(int id1, int id2) const
 {
-	if (id1 == id2)
-		return 0;
-
+	if (id1 == id2) return 0;
 	DDataItem *item1 = dataItem(id1);
 	DDataItem *item2 = dataItem(id2);
-	if (!item1 || !item2)
-		return 0;
+	if (!item1 || !item2) return 0;
 
-	if (item1->Parent() != item2->Parent())
-		return 0;
+	if (item1->Parent() != item2->Parent()) return 0;
 
 	if (item1->Parent() == DItem::NOITEM && item2->Parent() == DItem::NOITEM)
 	{
 		if (item1->countChildren() >= item2->countChildren())
 		{
-			if (item1->countChildren() > item2->countChildren())
-				return -1;
+			if (item1->countChildren() > item2->countChildren()) return -1;
 			return 0;
 		}
 		return 1;
 	}
 
 	DDataItem *dataitem_parent = dataItem(item1->Parent());
-	if (dataitem_parent->childIdx(id1) > dataitem_parent->childIdx(id2))
-		return -1;
-	else
-		return 1;
+	if (dataitem_parent->childIdx(id1) > dataitem_parent->childIdx(id2)) return -1;
+	else return 1;
 }
 
 void DDataControl::notifyChildren(int id)
 {
-	DDataItem * item = dataItem(id);
+	DDataItem* item = dataItem(id);
 	if (item)
 	{
 		// update the children positions
@@ -374,13 +377,14 @@ void DDataControl::printTree(QTextStream & s)
 
 	s<<"canvasfont=\""<<m_canvasFont.toString()<<"\" ";
 	s<<"picturesoncanvas=\""<<(int) m_picturesOnCanvas<<"\" ";
-	s<<"reversevideo=\""<<(int) m_reverseVideo<<"\" ";
 
-	s<<"showdirection=\""<<(int) m_showDirection<<"\" ";
+	s<<"generator_lasturl=\""<<m_generator_lasturl<<"\" ";
+	s<<"generator_lastgen=\""<<m_generator_lastgen<<"\" ";
+	s<<"generator_count=\""<<m_generator_count<<"\" ";
+	
 	s<<"showdirectionsize=\""<<m_showDirectionSize<<"\" ";
 	s<<"linkstyle=\""<<m_linkstyle<<"\" />\n";
 
-
 	QMapIterator<int, DItem*> it;
 	for (it = m_map.begin(); it != m_map.end(); ++it)
 	{
@@ -658,7 +662,7 @@ bool DDataControl::loadFromFile(const KURL & url)
 	{
 		DDataItem *item = dataItem(*it);
 		//kdWarning()<<"item is "<<item->Id()<<endl;
-		for (int i=0; i<item->ref_count(); i++)
+		for (unsigned int i=0; i<item->ref_count(); i++)
 		{
 			DRef *ref = item->ref_idx(i);
 			//kdWarning()<<"updating ref "<<item->Id()<<" "<<ref->m_ref<<endl;
@@ -675,8 +679,7 @@ bool DDataControl::saveToFile(const KURL & url)
 {
 	// check the name of the file to write to
 	QString fname = url.url();
-	if (! fname.endsWith(".kdi", false))
-		fname.append(".kdi");
+	if (! fname.endsWith(".kdi", false)) fname.append(".kdi");
 
 	// create a tremporary file in which the data will be written
 	KTempFile archfile;
@@ -738,16 +741,9 @@ int DDataControl::countItems()
 
 bool DDataControl::canGenerate()
 {
-	DDataItem *data = dataItem( rootID() );
-
-	if (!data)
-		return false;
-
-	if (data->countChildren() > 0)
-	{
-		return true;
-	}
-
+	DDataItem *rewt = dataItem( rootID() );
+	if (!rewt) return false;
+	if (rewt->countChildren() > 0) return true;
 	return false;
 }
 
@@ -854,14 +850,15 @@ void DDataControl::slotFileSpell()
 	m_spell->spell();
 }
 
-void DDataControl::setItemSelected(int val, DGuiView *view)
+void DDataControl::setItemSelected(int id, DGuiView *view)
 {
-	emit itemSelected(val, view);
+	m_itemselected = dataItem(id);
+	emit itemSelected(id, view);
 }
 
-void DDataControl::notifyItemModified(int val)
+void DDataControl::notifyItemModified(int id)
 {
-	emit itemChanged(val);
+	emit itemChanged(id);
 }
 
 bool DDataControl::requestRefChange(int orig, int dest, bool add, bool check)
@@ -884,16 +881,134 @@ bool DDataControl::requestRefChange(int orig, int dest, bool add, bool check)
 	if (add)
 	{
 		if ( item->isRef(dest) ) return false;
-		//kdWarning()<<"adding "<<dest<<" for orig "<<orig<<endl;
 		item->addRef(dest);
+		addDelta( new DDeltaLink(this, orig, dest, false, true) );
 	}
 	else
 	{
 		if ( !item->isRef(dest) ) return false;
 		item->rmRef(dest);
+		addDelta( new DDeltaLink(this, orig, dest, true, true) );
 	}
+	m_reflist.append( new QPoint(orig, dest) );
 	if (check) emit refChanged(orig, dest, add);
 	return true;
 }
 
+void DDataControl::addDelta(DDelta* delta)
+{
+	if (m_undoredo_locked)
+	{
+		delete delta;
+		return;
+	}
+
+	kdDebug()<<"adding delta "<<delta->Type()<<endl;
+	
+
+	m_redostack.clear();
+	emit redoState( false );
+	
+	if (! m_undostack.count()) newDelta();
+
+	DDeltaList* lst = m_undostack.top();
+	lst->insertDelta(delta);
+	
+	// set state to "undo is possible"
+	emit undoState( true );
+}
+
+void DDataControl::newDelta()
+{
+	DDeltaList* lst = m_undostack.top();
+
+	// no need to create a new delta list, the previous is empty
+	if (lst) if (!lst->count()) return;
+	
+	// limit the undo depth
+	if (m_undostack.count() > 25)
+	{
+		kdDebug()<<"undo stack contains more than 25 elements"<<endl;
+
+		// eliminates elements in excess, put the other elements in a temporary stack
+		QPtrStack<DDeltaList> st2; int count = 0; DDeltaList* dt;
+		while (dt = m_undostack.pop())
+		{
+			if (count<=25) st2.push(dt);
+			else delete dt;
+		}
+
+		// refill the undo stack
+		while (dt = st2.pop()) m_undostack.push(dt);
+	}
+
+	lst = new DDeltaList();
+	m_undostack.push(lst);
+}
+
+void DDataControl::slotUndo()
+{
+	m_undoredo_locked = true;
+
+	if (m_undostack.count())
+	{
+		DDeltaList *lst = m_undostack.pop();
+		lst->applyUndo();
+		m_redostack.push(lst);
+	}
+	
+	// check and set the state to "undo is possible"
+	emit undoState( m_undostack.count() );
+	emit redoState( m_redostack.count() );
+
+	m_undoredo_locked = false;
+}
+
+void DDataControl::slotRedo()
+{
+	m_undoredo_locked = true;
+	
+	if (m_redostack.count())
+	{
+		DDeltaList *lst = m_redostack.pop();
+		lst->applyRedo();
+		m_undostack.push(lst);
+	}
+	
+	// check and set the state to "redo is possible"
+	emit redoState( m_redostack.count() );
+	emit undoState( m_undostack.count() );
+
+	m_undoredo_locked = false;
+}
+
+void DDataControl::registerMapPositionChange(int id, int oldx, int oldy, int newx, int newy)
+{
+	if (!isRegistered(id))
+	{
+		kdWarning()<<"BUG : unregistered item at DDataControl::registerMapPositionChange"<<id<<endl;
+		return;
+	}
+	addDelta( new DDeltaMove(this, id, oldx, oldy, newx, newy) );
+}
+
+void DDataControl::registerMapColorChange(int id, int oldc, int newc, QColor oldcustcol, QColor newcustcol)
+{
+	if (!isRegistered(id))
+	{
+		kdWarning()<<"BUG : unregistered item at DDataControl::registerMapColorChange"<<id<<endl;
+		return;
+	}
+	addDelta( new DDeltaColor(this, id, oldc, newc, oldcustcol, newcustcol) );
+}
+
+void DDataControl::registerMapFlagChange(int id, int flag, bool add)
+{
+	if (!isRegistered(id))
+	{
+		kdWarning()<<"BUG : unregistered item at DDataControl::registerMapFlagChange"<<id<<endl;
+		return;
+	}
+	addDelta( new DDeltaFlag(this, id, flag, add) );
+}
 #include "DDataControl.moc"
